(******************************************************************************)
(*                                                                            *)
(*                      INVERSION DE MD* VIA SAT SOLVEUR                      *)
(*                                                                            *)
(*                      Projet Logique 2016 - Partie SAT                      *)
(*   Auteur, license, etc.                                                    *)
(******************************************************************************)

open Printf
open Param
       
type var = int
type literal =
  | Pos of var
  | Neg of var
type clause = literal list
type cnf = clause list

type formula =
  | Const of bool
  | Lit of literal
  | Not of formula
  | And of formula*formula
  | Or of formula*formula
  | Xor of formula*formula
  | Imply of formula*formula
  | Equiv of formula*formula

let rec displayFormula = function
  | Const b -> sprintf "%b" b
  | Lit l -> (match l with
	      | Pos d -> sprintf "+%d" d
	      | Neg d -> sprintf "-%d" d)
  | Not f -> sprintf "Not[%s]" (displayFormula f)
  | And (f1,f2) -> sprintf "[%s] /\\ [%s]" (displayFormula f1) (displayFormula f2)
  | Or (f1,f2) -> sprintf "(%s) \\/ (%s)" (displayFormula f1) (displayFormula f2)
  | Xor (f1,f2) -> sprintf "(%s) + (%s)" (displayFormula f1) (displayFormula f2)
  | Imply (f1, f2) -> sprintf "{%s} ==> {%s}" (displayFormula f1) (displayFormula f2)
  | Equiv (f1,f2) -> sprintf "{%s} <==> {%s}" (displayFormula f1) (displayFormula f2)
			     
let rec simple  f = f		(* [TODO] *)

let subst f tau = f		(* [TODO] *)
	   
let formulaeToCnf fl = [[]]	(* [TODO] *)
	      
let displayCnf cnf = ""		(* [TODO] *)

(*** TEST ***)
let dummyCNF =
  [[Pos 1;Pos 2;Pos 3];
   [Pos 2;Pos 3;Pos 4];
   [Pos 3;Pos 4;Neg 1]
  ]
let sat_solver = ref "./minisat"

(** Return the result of minisat called on [cnf] **)
let testCNF cnf = 
  let cnf_display = displayCnf cnf in
  let fn_cnf = "temp.out" in
  let oc = open_out fn_cnf in
  Printf.fprintf oc  "%s\n" cnf_display;
  close_out oc;
  let resc = (Unix.open_process_in
		(!sat_solver ^ " \"" ^ (String.escaped fn_cnf)
		 ^ "\"") : in_channel) in
  let resSAT = let acc = ref [] in
	       try while true do
		     acc := (input_line resc) :: !acc
		   done; ""
	       with End_of_file ->
		 close_in resc;
		 if List.length !acc = 0
		 then begin
		     log ~level:High "It seems that there is no executable called 'minisat' at top level.";
		     exit 0;
		   end
		 else String.concat "\n" (List.rev !acc) in
  close_in resc;
  List.hd (List.rev (Str.split (Str.regexp " +") resSAT))
	   
let test () =
  ()			     (* [TODO] *)
  (* Yout test using your functions here *)
